#define PERSONAL
#undef STAND_ALONE

#include <stdlib.h>
#ifdef __PUREC__
   #include <tos.h>
#else
   #include <osbind.h>
#endif
#include <string.h>

#ifndef _PROTO_H_
#include "proto.h"   /* Lattice C can have this precompiled */
#endif

#ifdef STAND_ALONE
  #include "..\headers\jpegd.h"
#else
  #include "jpegd.h"
#endif

#include "colour.h"


void *memalloc(long chunk, int change, long size);

#ifdef STAND_ALONE
int main(int argc, const char *argv[]);
#endif
int mytimer(JPDPB *);
int mydither(struct __jpdPB *mypb);
int dummyfunc1(struct __jpdPB *mypb);
int dummyfunc2(struct __jpdPB *mypb);
int dummyfunc3(struct __jpdPB *mypb);

static int counter;
static int line;
static int mode;

extern buffer_t buffer[];

extern globopts_t options;
extern char *errtext;
extern int Falcon;

extern long get_cookie(uchar_t *cname);
#ifndef __TURBOC__
extern int do_jpegd_asm();
extern int mydither_asm();
extern int mytimer_asm();
extern int dummyfunc1_asm();
extern int dummyfunc2_asm();
extern int dummyfunc3_asm();
#endif

#ifdef STAND_ALONE
int main(int argc, const char *argv[])
{
   char *fname;
   FILE *stream;
	
   fname = argv[1];

   if(fname == NULL) {
      printf("No parameter passed\n");
      return(-1);
   }

   return(dsp_load(fname));
}
#endif

#ifdef PERSONAL      /* Use for myself only */
int dsp_load(char *fname, int colour_mode, int *width, int *height)
{
   JPD_functab *functab;
   JPDPB *mypb;
   int handle;
   long length, pbsize, tempw, temph;
   uchar_t *start, *linebuf;
   FILE *stream;
   long maxi;
   int ret;

   mode = colour_mode;

   if (colour_mode == 2)   /* No support for colour reduction yet */
      return(-1);

   if (!Falcon) {
      if (options.xtra.debug)
         printf("Not a Falcon030!\n");
      return(-1);
   }

   if ((functab = (JPD_functab *)get_cookie("_JPD")) == (void *)-1L) {
      if (options.xtra.debug)
         printf("JPEGD driver not found\n");
      return(-1);
   }

#ifdef __TURBOC__
   pbsize = functab->JPD_getPBsize();
#else
   pbsize = do_jpegd_asm(functab->JPD_getPBsize, NULL);
#endif
   
   if ((stream = fopen(fname, "rb")) == NULL) {
      printf("Cannot open file\n");
      return(EGIFEOF);
   }

#ifdef __GNUC__
   stream->_flag |= _IOBIN;   /* The GNU libraries don't like "rb" */
#endif

   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, 0L, SEEK_SET);

   if (length + pbsize + 4 > buffer[options.mem.no_rasters].size)
#if 0
      return(EGIFBIG);
#endif
      return(-1);

   start = buffer[options.mem.no_rasters].address;
      	
   fread(start, 1L, length, stream);

   buffer[options.mem.no_rasters].width = buffer[options.mem.no_rasters].height = 0;
	
   fclose(stream);

/* step 1 allocate the parameter block */

#if 0
   mypb = Malloc(pbsize = functab->JPD_getPBsize());
#endif

#if 0
   if ((mypb = (JPDPB *)memalloc(pbsize, -1, 0L)) == NULL) {
      return(EGIFBIG);
   }
#endif
   mypb = (JPDPB *)((long)(start + length + 2) & 0xfffffffeL);

   memset(mypb, 0, pbsize);

/* step 2 open our "JPD station" */

#ifdef __TURBOC__
   functab->JPD_open(mypb);
#else
   do_jpegd_asm(functab->JPD_open, mypb);
#endif

   mypb->comp_data = start;
   mypb->comp_size = length;
#ifdef __TURBOC__
   functab->JPD_getJPGinfo(mypb);
#else
   do_jpegd_asm(functab->JPD_getJPGinfo, mypb);
#endif

   tempw = mypb->image_w + 15;
   tempw &= ~15;
   temph = mypb->image_h + 7;
   temph &= ~7;
	
   mypb->magic12 = 3;        /* 24 bit */
   mypb->magic14 = 3;        /* # of bytes/pixel */
   mypb->ditherflag = -1;     /* -1 - dither */
#ifdef __TURBOC__
   functab->JPD_calcpixels(mypb);
#else
   do_jpegd_asm(functab->JPD_calcpixels, mypb);
#endif

#if 0
   linebuf = Malloc(tempw * 3); /* 3 represents bytes/pixel */
   mypb->decomp_data1 = linebuf;
   mypb->planes = 8;
#endif

#ifdef __TURBOC__
   mypb->dither = mydither;
   mypb->dummyfun1 = dummyfunc1;
   mypb->dummyfun2 = dummyfunc2;
   mypb->dummyfun3 = dummyfunc3;
   mypb->timer = mytimer;
#else
   mypb->dither = mydither_asm;
   mypb->dummyfun1 = dummyfunc1_asm;
   mypb->dummyfun2 = dummyfunc2_asm;
   mypb->dummyfun3 = dummyfunc3_asm;
   mypb->timer = mytimer_asm;
#endif

#if 0
   mypb->decomp_data = Malloc(tempw * temph * 3); /* 3 represents bytes/pixel */
#endif

   counter = 0;
   line = 0;

   maxi = buffer[0].size / 3;

   if (options.colours.mode == 3) {
      buffer[1].address = buffer[0].address + maxi;
      buffer[2].address = buffer[0].address + maxi * 2;
      buffer[1].size = maxi;
      buffer[2].size = buffer[0].size - 2 * maxi;
      buffer[0].size = maxi;
      buffer[0].type = _red;
      buffer[1].type = _green;
      buffer[2].type = _blue;
      buffer[1].width = buffer[2].width = buffer[0].width = mypb->image_w;
      buffer[1].height = buffer[2].height = buffer[0].height = mypb->image_h;
#if 0
      for(i = 0;i < buffer[0].height;i++) {
         image[0][i] = rast;
         image[1][i] = rast + maxi;
         image[2][i] = rast + maxi * 2;
         rast += buffer[0].width;
      }
#endif
   } else {
      buffer[0].type = _grey;
      buffer[0].width = mypb->image_w;
      buffer[0].height = mypb->image_h;
   }

#ifdef __TURBOC__
   ret = functab->JPD_decode(mypb);
	
   functab->JPD_close(mypb);
#else
   ret = do_jpegd_asm(functab->JPD_decode, mypb);
	
   do_jpegd_asm(functab->JPD_close, mypb);
#endif

   *width = mypb->image_w;
   *height = mypb->image_h;

   if (ret)
      return(-1);    /* Try normal loader instead */
   else
      return(0);
}

int dummyfunc1(struct __jpdPB *mypb)
{
   if (options.xtra.debug)
      printf("Dummy 1 called!\n");
   return(0);
}

int dummyfunc2(struct __jpdPB *mypb)
{
   if (options.xtra.debug)
      printf("Dummy 2 called!\n");
   return(0);
}

int dummyfunc3(struct __jpdPB *mypb)
{
   if (options.xtra.debug)
      printf("Dummy 3 called!\n");
   return(0);
}

int mydither(struct __jpdPB *mypb)
{
   int x, y, sum, addpos;
   unsigned char *red, *green, *blue, *all;

   if (options.xtra.debug)
      printf("'Dithering' (%ld, %d, %d)\n",
             (long)mypb->outbuffer, mypb->dummy4[0], mypb->dummy4[1]);

   red   = buffer[0].address + (long)line * mypb->image_w;
   green = buffer[1].address + (long)line * mypb->image_w;
   blue  = buffer[2].address + (long)line * mypb->image_w;
   all = mypb->outbuffer;

   addpos = (((mypb->image_w + 15) & 0xfff0) - mypb->image_w) * 3;

   if (mode == 3) {
      for(x = mypb->dummy4[1] - 1;x >= 0;x--) {
         for(y = mypb->image_w - 1;y >= 0;y--) {
            *red++ = *all++;
            *green++ = *all++;
            *blue++ = *all++;
         }
         all += addpos;
      }
   } else {
      for(x = mypb->dummy4[1] - 1;x >= 0;x--) {
         for(y = mypb->image_w - 1;y >= 0;y--) {
#if 0
            sum = *all++;      /* Should not really add up, only for test! */
            sum += *all++;
            sum += *all++;
            *red++ = sum / 3;   /* Actually grey of course */
#endif
            sum = *all++ * 77;
            sum += *all++ * 151;
            sum += *all++ * 28;
            *red++ = sum / 256;   /* Actually grey of course */
         }
         all += addpos;
      }
   }
   line += mypb->dummy4[1];
   return(0);
}

int mytimer(JPDPB *copyPB){
   char s[4];

   counter++;
#if 0
   printf("Image decoding %d\n", (100 * counter) / copyPB->zones);
#endif
   return(0);
}

#else    /* Used for distributed versions of MGIF */

int dsp_load(char *fname, int colour_mode, int *width, int *height)
{
   return(-1);
}
int mytimer(JPDPB *mypb)
{
   return(0);
}

int mydither(struct __jpdPB *mypb)
{
   return(0);
}

int dummyfunc1(struct __jpdPB *mypb)
{
   return(0);
}

int dummyfunc2(struct __jpdPB *mypb)
{
   return(0);
}

int dummyfunc3(struct __jpdPB *mypb)
{
   return(0);
}

#endif
