#include <tos.h>
#include <ext.h>
#include <stdio.h>
#include <stdlib.h>
#include <falcon.h>
#include <VIDEO.H>
#include "_jpd.h"

#include "def.h"
#include "struct.h"
#include "cpeg.h"
#include "asm.h"
#include "incbin.h"
extern long ssp;
extern void *ts;

void nop(void) 0x4E71;

void interpolate(PICTURE *pic, VIDEL_INFO *vid_info){
	double kx, ky, k, xpos, ypos;
	int *src, *dst;
	unsigned long scanline, xstart;
	unsigned int /* red, green, blue, */ i, j;
	
	kx=(double)pic->dx/vid_info->dx;
	ky=(double)pic->dy/vid_info->dy;
	if(kx>ky) k=kx; else k=ky;
	if(k<=1) return;
	
	scanline=(pic->dx+pic->offset)<<1;
	pic->dx/=k;
	pic->dx&=~1;
	pic->dy/=k;
	pic->dy&=~1;
	pic->offset=0;
	
	dst=pic->start;
	ypos=0;
		
	for(i=0;i<pic->dy;i++){
		xstart=(long)pic->start+(scanline*(long)ypos);
		xpos=0;
		ypos+=k;
		
		for(j=0;j<pic->dx;j++){
			src=(int *)(xstart+((long)xpos<<1));
			xpos+=k;
			*dst++=*src;
		}
	}
}

void halfsize(PICTURE *pic){
	unsigned long scanline;
	
	Dsp_ExecProg(halfsize_beg, halfsize_size, ABILLITY);
	*(char *)0xFFFFA200L=128;      /* init host port */
	
	scanline=(pic->dx+pic->offset)<<1;
	pic->dx>>=1;
	pic->dx&=~1;
	pic->dy>>=1;
	pic->dy&=~1;
	pic->offset=0;
	
	hs_asm(pic, scanline);
	
}

void wait_mouse(void){
	MY_MOUSE mouse;
	
	do{
		get_mouse(&mouse);
	} while(mouse.mb_left==0);
}

int show_picture(PICTURE *picture, VIDEL_INFO *videl_info, VIDL *videl, MY_MOUSE *mouse){
	int hw_dx_enable=0, hw_dy_enable=0;
	long tmp_long;
	
	if(picture->dx>videl_info->dx) hw_dx_enable=picture->dx-videl_info->dx+1;
	if(picture->dy>videl_info->dy) hw_dy_enable=picture->dy-videl_info->dy+1;
	mouse->mb_left=0;
	set_mouse(hw_dx_enable, hw_dy_enable);
	
	while(mouse->mb_left==0){
		get_mouse(mouse);
		if((mouse->mb_right!=0 && mouse->mb_right_touched!=0) && (picture->dx>videl_info->dx || picture->dy>videl_info->dy)) break;
		if(mouse->mx>hw_dx_enable) mouse->mx=hw_dx_enable-1;
		if(mouse->my>hw_dy_enable) mouse->my=hw_dy_enable-1;
		
		tmp_long=(((long)mouse->my>>1)*((long)picture->dx+(long)picture->offset)*4L)+(((long)mouse->mx>>1)*4L)+(long)picture->start;
		vsync();
		if(videl_info->skip_line!=0) vsync();
		set_phys((void *)tmp_long);			
		den_black(TRUE, videl);
	}
	
	if(mouse->mb_left!=0) return FALSE; else return TRUE;
}

int status(JPDPB *copyPB){
	static int x=10, flag=FALSE;
	static float time, counter=0;
	
	if(flag==FALSE){
		time=copyPB->zones/(float)9;
		flag=TRUE;
	}
	
	if(counter++>time){
		print(x,5,"*",ts);
		counter-=time;
		x++;
	}
	return 0;
}

void super(void){
	if(Super((void *)1L) == 0L)
		ssp = Super((void *)0L);
	else
		ssp = 0L;
}

void user(void){
	if(ssp)
		Super((void *)ssp);
}

int load(const char *fname, FINFO *fipek){
	int handle;
	
	handle=(int)Fopen(fname, FO_READ);
	if(handle<0) return 1;
	fipek->size=Fseek(0L, handle, 2);
	Fseek(0L, handle, 0);
	
	fipek->start=malloc(fipek->size);
	if(fipek->start==0) return 2;
	
	if(Fread(handle, fipek->size, fipek->start)!=fipek->size) return 1;
	if(Fclose(handle)!=0) return 1;
	return 0;
}

int getcookie(long target, long *value){
	COOKIE *cookie_ptr;
	
	super();	
	cookie_ptr = *(COOKIE **)0x5A0;
	user();
	
	if(cookie_ptr != NULL){
		do{
			if(cookie_ptr->cookie == target){
				if(value != NULL)
					*value = cookie_ptr->value;
				return 1;
			}
		}while((cookie_ptr++)->cookie != 0L);
	}
	return 0;
}

void make_x_tab(void){
	long i,j=0;
	
	for(i=0;i<XRES/8;i+=2){
		*(x_tab+i)=j;
		*(x_tab+i+1)=j+1;
		j+=8;
	}
}

void make_y_tab(void){
	long i,j=0;
	
	for(i=0;i<YRES/8;i++){
		*(y_tab+i)=j;
		j+=(XRES/2*8);
	}
}