#include <tos.h>
#include <ext.h>
#include <stdio.h>
#include <stdlib.h>
#include <falcon.h>
#include <VIDEO.H>
#include "_jpd.h"

#include "def.h"
#include "struct.h"
#include "cpeg.h"
#include "asm.h"
#include "incbin.h"


long ssp;
void *ts;

void main(const char *argv[]){
	FINFO	script_finfo, videl_finfo, jpeg_finfo;
	SCRIPT script_pointers;
	
	const char fname_inf[]="data\\cpeg.inf", *fname_jpeg=argv[1];
	char string[18];
	int display, exit;
	
	VIDL old_videl, *text_videl;
	
	PICTURE picture;
	VIDEL_INFO videl_info;
	
	long old_pal[2], border=0;	
	void *old_phys, *text_screen;
	
	MY_MOUSE mouse;
	
	JPDPB *mypb;
	long pbsize;
	JPD_functab *functab;
			
	
	super();
	
	display=monitor();
	switch(display){
		case 0:{
			Cconws("Monochrome monitors are not suported.");
			Crawcin();
			return;
		}
		case 2: text_videl=&mtext; break;
		default: text_videl=&ttext;
	}
	
	ts=text_screen=malloc((long)SCREEN_SIZE);
	if(text_screen==NULL){
		Cconws("Out of memory!");
		Crawcin();
		return;
	}
	
	clear_memory(text_screen, (long)SCREEN_SIZE);
	make_x_tab();
	make_y_tab();
	init_mouse();
	
	/* Spara och ndra upplsning */
	
	vsync();
	get_videl(&old_videl);
	old_phys=get_phys();
	get_pal(0,2,old_pal);
	vsync();
	set_videl(text_videl);
	den_black(FALSE, text_videl);
	set_phys(text_screen);
	set_pal(0,2,shell_pal);
	delay(100);
	print(0,0,header,text_screen);
	print(0,3,"Loading...",text_screen);
	vsync();
	den_black(TRUE, text_videl);
	
	do {
		/* Load script */
		
		switch(load(fname_inf, &script_finfo)){
			case 0: exit=FALSE; break;
			case 1:{
				print(0,4,"Error when reading CPEG.INF",text_screen);
				wait_mouse();
				exit=TRUE;
			} break;
			case 2:{
				print(0,4,"Out of memory!",text_screen);
				wait_mouse();
				exit=TRUE;
			} break;
		} if(exit==TRUE) break;
		
		evaluate_script((char *)script_finfo.start, &script_pointers);
		
		/* is there a _JPD cockie */
		
		if(!getcookie('_JPD',(long *)&functab)){
			Pexec((int)0, script_pointers.jpegd, "", (void *)0);
			if(!getcookie('_JPD',(long *)&functab)){
				print(0,4,"Error when searching for _JPD cockie.",text_screen);
				wait_mouse();
				break;
			}
		}
		
		/* Load *.VID */
		
		switch(display){
			case 2:{
				switch(load(script_pointers.multi, &videl_finfo)){
					case 0: exit=FALSE; break;
					case 1:{
						print(0,4,"Error when reading *.VID file.",text_screen);
						wait_mouse();
						exit=TRUE;
					} break;
					case 2:{
						print(0,4,"Out of memory!",text_screen);
						wait_mouse();
						exit=TRUE;
					} break;
				}
			} break;
			default:{
				switch(load(script_pointers.tv, &videl_finfo)){
					case 0: exit=FALSE; break;
					case 1:{
						print(0,4,"Error when reading *.VID file.",text_screen);
						wait_mouse();
						exit=TRUE;
					} break;
					case 2:{
						print(0,4,"Out of memory!",text_screen);
						wait_mouse();
						exit=TRUE;
					} break;
				}
			}
		} if(exit==TRUE) break;
		
		get_videl_info(&videl_info, videl_finfo.start);
		if(videl_info.hi_colour==FALSE){
			print(0,4,"You must use a HiColour VIDL file.",text_screen);
			wait_mouse();
			break;
		} 
		
		/* Load *.JPG */
		
		switch(load(fname_jpeg, &jpeg_finfo)){
			case 0: exit=FALSE; break;
			case 1:{
				print(0,4,"Error when reading *.JPG",text_screen);
				wait_mouse();
				exit=TRUE;
			} break;
			case 2:{
				print(0,4,"Out of memory!",text_screen);
				wait_mouse();
				exit=TRUE;
			} break;
		} if(exit==TRUE) break;
		
		
/* step 1 allocate the parameter block */
		
		mypb=malloc(pbsize=functab->JPD_getPBsize());
		if(mypb==NULL){
			print(0,4,"Out of memory!",text_screen);
			wait_mouse();
			break;
		}
		clear_memory(mypb, pbsize);
		
/* step 2 open our "JPD station" */
		
		functab->JPD_open(mypb);
		
		mypb->comp_data=jpeg_finfo.start;
		mypb->comp_size=jpeg_finfo.size;
		if(functab->JPD_getJPGinfo(mypb)!=0){
			print(0,4,"This is not a valid JPEG file.",text_screen);
			wait_mouse();
			break;
		}
		
		mypb->magic12=3;
		mypb->magic14=2;
		mypb->ditherflag=0;
		functab->JPD_calcpixels(mypb);
		
		sprintf(string, "Size: %u*%u", mypb->image_w, mypb->image_h);
		print(0,4,string,text_screen);
		
		picture.dx = mypb->image_w + 15;
		picture.dx &= ~15;
		picture.dy = mypb->image_h + 7;
		picture.dy &= ~7;
		
		picture.start=mypb->decomp_data=malloc((long)picture.dx*picture.dy*2);
		if(picture.start==NULL){
			/* Mysko bug!!! */		
			make_x_tab();
			/*make_y_tab();*/
			print(0,5,"Out of memory!",text_screen);
			wait_mouse();
			break;
		}
		
		picture.offset=picture.dx;
		picture.dx=mypb->image_w;
		picture.dx &= ~1;
		picture.offset-=picture.dx;
		picture.dy=mypb->image_h;
		picture.dy &= ~1;
		
		mypb->timer=status;
		Dsp_ExecProg(jpegddsp_beg, (long)804 /*jpegddsp_size*/, ABILLITY);
		/* Mysko bug!!! */		
		make_x_tab();
		/*make_y_tab();*/
		print(0,5,"Decoding [........]",text_screen);
		functab->JPD_decode(mypb);
		functab->JPD_close();
		
		/* Show picture */
		
		/*get_videl_info(&videl_info, videl_finfo.start);*/
		/*interpolate(&picture,&videl_info);*/
		
		modify_videl(&picture, videl_finfo.start, (long)&videl_info);
		
		vsync();
		set_videl(videl_finfo.start);
		den_black(FALSE, videl_finfo.start);
		set_pal(0,1,&border);
		delay(100);
		
		while(show_picture(&picture, &videl_info, videl_finfo.start, &mouse)!=0){
			/*half_size_calculate(&picture);
			get_videl_info(&videl_info, videl_finfo.start);*/
			
			vsync();
			den_black(FALSE, videl_finfo.start);
			/*half_size(&picture);*/
			halfsize(&picture);
			get_videl_info(&videl_info, videl_finfo.start);
			modify_videl(&picture, videl_finfo.start, (long)&videl_info);
			
			vsync();
			set_videl(videl_finfo.start);
			den_black(FALSE, videl_finfo.start);
		}
		
	} while(FALSE);
	free(text_screen);
	free(script_finfo.start);
	free(videl_finfo.start);
	free(jpeg_finfo.start);
	free(mypb);
	free(picture.start);
	
	vsync();
	set_videl(&old_videl);
	den_black(FALSE,&old_videl);
	set_phys(old_phys);
	set_pal(0,2,old_pal);
	delay(100);
	vsync();
	den_black(TRUE,&old_videl);
	
	restore_mouse();
	user();
	return;
}